"""
Resources for myanmartools.

This peckage contains:

- `zawgyiUnicodeModel.dat` - parameters of Zawgyi detector
- `compatability.tsv` - Zawgyi probabilities and input strings
  resulting from Java implementation of Zawgyi detector
"""
